<?php
namespace OPAY\PaymentGateway\Helper;

use OPAY\PaymentGateway\Model\Config\Source\SignatureType;
use OPAY\PaymentGateway\Gateway\Request\DataRequest;
use OPAY\PaymentGateway\Helper\OpayGatewayFactory;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\UrlInterface;
use Magento\Sales\Model\Order;
use OPAY\PaymentGateway\Model\Ui\ConfigProvider;
use Magento\Framework\App\ResponseFactory;
use Magento\Checkout\Model\DefaultConfigProvider;

class RequestHandler
{
    /**
     * @var \OpayGateway
     */
    private $opayGateway;

    /**
     * @var \OPAY\PaymentGateway\Gateway\Request\DataRequest
     */
    private $requestBuilder;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    private $request;

    /**
     * @var \Magento\Framework\App\ResponseInterface
     */
    private $response;

    /**
     * @var \Magento\Framework\App\UrlInterface
     */
    private $url;

    /**
     * @var \Magento\Sales\Model\Order
     */
    private $order;

    /**
     * @var \Magento\Framework\App\ResponseFactory
     */
    private $responseFactory;

    /**
     * @var \Magento\Checkout\Model\DefaultConfigProvider
     */
    private $defaultConfigProvider;

    /**
     * Constructor
     *
     * @param DataRequest $dataRequest
     * @param OpayGatewayFactory $opayGatewayFactory
     * @param RequestInterface $request
     * @param ResponseInterface $response
     * @param UrlInterface $url
     * @param Order $order
     * @param ResponseFactory $responseFactory
     * @param DefaultConfigProvider $defaultConfigProvider
     */
    public function __construct(
        DataRequest $dataRequest, 
        OpayGatewayFactory $opayGatewayFactory, 
        RequestInterface $request,
        ResponseInterface $response,
        UrlInterface $url,
        Order $order,
        ResponseFactory $responseFactory,
        DefaultConfigProvider $defaultConfigProvider
    ) {
        $this->requestBuilder = $dataRequest;
        $this->opayGateway = $opayGatewayFactory->create();
        $this->request = $request;
        $this->response = $response;
        $this->url = $url;
        $this->order = $order;
        $this->responseFactory = $responseFactory;
        $this->defaultConfigProvider = $defaultConfigProvider;
    }

    /**
     * {@inheritdoc}
     *
     * @see \Magento\Framework\App\ActionInterface
     */
    public function handleCallback()
    {
        try {
            // Try to form request data from request object
            $requestData = $this->requestBuilder->build(array(
                'request' => $this->request
            ));
        } catch (\Exception $exc) {
            // TODO log exception
            $this->responseFactory->create()->setRedirect($this->url->getBaseUrl())->sendResponse();
            exit();
        }

        $order = $this->order->loadByIncrementId($requestData['order_nr']);
        $payment = $order->getPayment();

        // Checking if order payment is ours
        if ($payment->getMethodInstance()->getCode() != ConfigProvider::CODE) {
            // TODO log error
            $this->responseFactory->create()->setRedirect($this->url->getBaseUrl())->sendResponse();
            exit();
        }

        /** @var Magento\Framework\App\Response\Http */
        $response = $this->response;

        if (!in_array($order->getState(), array(
            $order::STATE_NEW,
            $order::STATE_PENDING_PAYMENT
        ))) {
            return $response->setBody('OK');
        }

        try {
            if ($requestData['status'] === '1') {
                $amount = $requestData['p_amount'] / 100;
                $payment->registerCaptureNotification($amount);
                $order->save();
                $response->setBody('OK');
            } elseif ($requestData['status'] === '0') {
                if ($order->canCancel()) {
                    $comment = __('Registered notification about failed payment.');
                    $order->cancel();
                    $order->addStatusHistoryComment($comment);
                    $order->save();
                }
                $response->setBody('OK');
            } else {
                $response->setBody('NOT OK');
            }
        } catch (\Exception $exc) {
            // TODO log exception
            $response->setBody('NOT OK');
        }

        return $response;
    }
    
    public function handleUserRedirect()
    {
        try {
            // Try to form request data from request object
            $requestData = $this->requestBuilder->build(array(
                'request' => $this->request
            ));
        } catch (\Exception $exc) {
            // TODO log exception
            $this->responseFactory->create()->setRedirect($this->url->getBaseUrl())->sendResponse();
            exit();
        }

        $order = $this->order->loadByIncrementId($requestData['order_nr']);
        $payment = $order->getPayment();

        // Checking if order payment is ours
        if ($payment->getMethodInstance()->getCode() != ConfigProvider::CODE) {
            // TODO log error
            $this->responseFactory->create()->setRedirect($this->url->getBaseUrl())->sendResponse();
            exit();
        }

        if (in_array($order->getState(), array(
            $order::STATE_NEW,
            $order::STATE_PENDING_PAYMENT
        ))) {
            if ($requestData['status'] === '1') {
                try {
                    $amount = $requestData['p_amount'] / 100;
                    $payment->registerCaptureNotification($amount);
                    $order->save();
                } catch (\Exception $exc) {
                    // TODO log exception
                }
            }
            // Note: status 2 goes to success page
        }

        $successPageUrl = $this->defaultConfigProvider->getDefaultSuccessPageUrl();
        $this->responseFactory->create()->setRedirect($successPageUrl)->sendResponse();
        exit();
    }
}