<?php
namespace OPAY\PaymentGateway\Model\Ui;

use Magento\Checkout\Model\ConfigProviderInterface;
use Magento\Checkout\Model\Session;
use Magento\Framework\Locale\ResolverInterface;
use Magento\Framework\UrlInterface;
use Magento\Payment\Helper\Data as PaymentHelper;
use OPAY\PaymentGateway\Helper\OpayGatewayFactory;
use OPAY\PaymentGateway\Model\Config\Source\ShowPaymentMethods;

/**
 * Class ConfigProvider
 */
final class ConfigProvider implements ConfigProviderInterface
{
    const CODE = 'opay_gateway';

    /**
     * @var \OpayGateway
     */
    private $opayGateway;

    /**
     * @var \Magento\Framework\Locale\ResolverInterface
     */
    private $localeResolver;

    /**
     * @var \Magento\Checkout\Model\Session;
     */
    private $checkoutSession;

    /**
     * @var \Magento\Payment\Model\MethodInterface;
     */
    private $method;

    /**
     * @var \Magento\Framework\UrlInterface
     */
    private $url;

    /**
     * @var array Stores settings retrieved from api
     */
    private $opaySettings;

    /**
     * @var array Stores channels info retrieved from api
     */
    private $apiChannelsResponse;

    /**
     * Constructor
     */
    public function __construct(
        PaymentHelper $paymentHelper,
        OpayGatewayFactory $opayGatewayFactory,
        Session $checkoutSession,
        ResolverInterface $localeResolver,
        UrlInterface $url
    ) {
        $this->method = $paymentHelper->getMethodInstance(self::CODE);
        $this->checkoutSession = $checkoutSession;
        $this->opayGateway = $opayGatewayFactory->create();
        $this->localeResolver = $localeResolver;
        $this->url = $url;
    }

    /**
     * {@inheritdoc}
     *
     * @see \Magento\Checkout\Model\ConfigProviderInterface
     */
    public function getConfig()
    {
        $config = array(
            'payment_gateway_name' => $this->getPaymentGatewayName(),
            'show_payment_methods' => $this->method->getConfigData('show_payment_methods'),
            'icon_height'          => $this->method->getConfigData('icon_height')
        );

        if (in_array($this->method->getConfigData('show_payment_methods'), array(
            ShowPaymentMethods::IN_CHECKOUT,
            ShowPaymentMethods::IN_CHECKOUT_GROUPED
        ))) {
            $channelsResult = $this->getChannels();
            if (!empty($channelsResult)) {
                $channelsResult = $this->prepareChannelsResult($channelsResult);
            } else {
                $channelsResult = null;
            }
            $config['channels'] = $channelsResult;
        }

        return array(
            'payment' => array(
                self::CODE => $config
            )
        );
    }

    /**
     * Prepares channels list result retrieved from api for client
     * side. It converts some array keys to indexes. So they could be
     * converted to js arrays.
     */
    private function prepareChannelsResult(array $result)
    {
        foreach ($result as $groupName => $group) {
            foreach ($group['channels'] as $channelName => $channel) {
                $group['channels'][$channelName]['id'] = $channelName;
            }
            $result[$groupName]['id'] = $groupName;
            $result[$groupName]['channels'] = array_values($group['channels']);
        }
        return array_values($result);
    }

    /**
     * Gets channels info from api
     *
     * @return array
     */
    private function getChannels()
    {
        if (empty($this->apiChannelsResponse)) {

            $quote = $this->checkoutSession->getQuote();
            $billingAddress = $quote->getBillingAddress();
            $total = $quote->getGrandTotal();
            $currency = $quote->getQuoteCurrencyCode();
            $locale = $this->localeResolver->getLocale();
            $language = strtoupper(substr($locale, 0, 2));

            $params = array(
                'website_id'      => $this->method->getConfigData('website_id'),
                'order_nr'        => $this->checkoutSession->getQuoteId(),
                'redirect_url'    => $this->url->getUrl('opay/pay/userredirect'),
                'web_service_url' => $this->url->getUrl('opay/pay/callback'),
                'standard'        => 'opay_8.1',
                'country'         => $billingAddress->getCountryId(),
                'language'        => $language,
                'amount'          => intval(number_format(($total * 100), 0, '', '')),
                'currency'        => $currency
            );

            if ($this->method->getConfigData('test_mode')) {
                $params['test'] = $this->method->getConfigData('user_id');
            }

            try {
                $response = $this->opayGateway->webServiceRequest(
                    'https://gateway.opay.lt/api/listchannels/', // TODO put somewhere else this url
                    $this->opayGateway->signArrayOfParameters($params)
                );

                // TODO Log errors here
                $this->apiChannelsResponse = $response;

            } catch (\OpayGatewayException $e) {
                // TODO log exception
            }

        }

        if (isset($this->apiChannelsResponse['response']['result'])) {
            return $this->apiChannelsResponse['response']['result'];
        } else {
            return array();
        }
    }

    /**
     * @return string
     */
    private function getPaymentGatewayName()
    {
        return $this->getSettings('payment_gateway_name');
    }

    /**
     * Gets settings from api
     *
     * @return string|array
     */
    private function getSettings($field = '')
    {
        if (empty($this->opaySettings)) {

            $locale = $this->localeResolver->getLocale();
            $language = strtoupper(substr($locale, 0, 2));
            $billingAddress = $this->checkoutSession->getQuote()->getBillingAddress();

            $params = array(
                'service_name' => 'getSettings',
                'website_id'   => $this->method->getConfigData('website_id'),
                'language'     => $language,
            );

            if ($countryId = $billingAddress->getCountryId()) {
                $params['country'] = $countryId;
            }

            $response = array();

            try {
                $response = $this->opayGateway->webServiceRequest(
                    'https://gateway.opay.lt/api/websites/', // TODO put somewhere else this url
                    $this->opayGateway->signArrayOfParameters($params)
                );
            } catch (\OpayGatewayException $e) {
                // TODO log exception
            }

            if (!empty($response['response']['result'])) {
                $this->opaySettings = $response['response']['result'];
            }
        }

        if ($field != '') {
            return (isset($this->opaySettings[$field])) ? $this->opaySettings[$field] : '';
        } else {
            return (!empty($this->opaySettings)) ? $this->opaySettings : array();
        }
    }
}